@@ -1,16 +1,17 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*
-"""Lispy missing batteries for Python.
+"""Supercharge your Python with parts of Lisp and Haskell.
 
 See ``dir(unpythonic)`` and submodule docstrings for more.
+
+If you have MacroPy installed, see also ``unpythonic.syntax``.
 """
 
-__version__ = '0.10.4'
+__version__ = '0.12.0'
 
 from .amb import *
 from .arity import *
 from .assignonce import *
-from .dynscope import *
+from .dynassign import *
 from .ec import *
 from .fold import *
 from .fploop import *
