@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """Folds, scans (lazy partial folds), and unfold."""
 
@@ -9,9 +8,10 @@ __all__ = ["scanl", "scanr", "scanl1", "scanr1",
 
 from functools import partial
 from itertools import zip_longest
-from collections import deque
+#from collections import deque
 
-from unpythonic.it import first, last, rev
+#from .it import first, last, rev
+from .it import last, rev
 
 # Require at least one iterable to make this work seamlessly with curry. We take
 # this approach with any new function families the standard library doesn't provide.
@@ -62,9 +62,10 @@ def scanr(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
 
         from operator import add
         assert tuple(scanl(add, 0, range(1, 5))) == (0, 1, 3, 6, 10)
-        assert tuple(scanr(add, 0, range(1, 5))) == (10, 9, 7, 4, 0)
+        assert tuple(scanr(add, 0, range(1, 5))) == (0, 4, 7, 9, 10)
 
-    The ordering of the output matches Haskell's ``scanr``.
+    **CAUTION**: The ordering of the output is different from Haskell's ``scanr``;
+    we yield the results in the order they are computed (via a linear process).
 
     For multiple input iterables, the notion of *corresponding elements*
     is based on syncing the **left** ends.
@@ -83,8 +84,8 @@ def scanr(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
         # rfoldl: reverse each input, then left-fold
         assert rfoldl(append_tuple, (), (1, 2, 3), (4, 5)) == ((3, 5), (2, 4))
     """
-    # Linear process: sync left ends; reverse; scanl into a deque.
-    # (Flat is better than nested; applied to the call stack.)
+    # Linear process: sync left ends; reverse; scanl.
+    # (Flat is better than nested, also for the call stack.)
     #
     # The implicit tuple(...) in rev(...) may seem inelegant, but it doesn't
     # really matter whether we keep the data in stack frames (like in the
@@ -95,13 +96,23 @@ def scanr(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
     xss = rev(z(iterable0, *iterables))
     if init_from_lastx:
         init = next(xss)[0]  # let StopIteration propagate
+
+#    # left-append into a deque to get same output order as in Haskell
+#    acc = init
+#    que = deque()
+#    que.appendleft(acc)
+#    for xs in xss:
+#        acc = proc(*(xs + (acc,)))
+#        que.appendleft(acc)
+#    yield from que
+
+    # to be more rackety/pythonic: yield results in the order they're computed
     acc = init
-    que = deque()
-    que.appendleft(acc)  # left-append to get same output order as in Haskell
+    yield acc
     for xs in xss:
         acc = proc(*(xs + (acc,)))
-        que.appendleft(acc)
-    yield from que
+        yield acc
+
 
 # Equivalent recursive process:
 #def scanr(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
@@ -183,8 +194,10 @@ def foldl(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
 
 def foldr(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
     """Dual of foldl; fold from the right."""
-    return first(scanr(proc, init, iterable0, *iterables,
-                       longest=longest, fillvalue=fillvalue))
+    # if using the haskelly result ordering in scanr, then first(...);
+    # if ordering results as they are computed, then last(...)
+    return last(scanr(proc, init, iterable0, *iterables,
+                      longest=longest, fillvalue=fillvalue))
 
 def reducel(proc, iterable, init=None):
     """Foldl for a single iterable, with optional init.
@@ -199,7 +212,9 @@ def reducer(proc, iterable, init=None):
 
     If ``init is None``, use the last element from the iterable.
     """
-    return first(scanr1(proc, iterable, init))
+    # if using the haskelly result ordering in scanr, then first(...);
+    # if ordering results as they are computed, then last(...)
+    return last(scanr1(proc, iterable, init))
 
 def rscanl(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
     """Reverse each input, then scanl.
@@ -318,177 +333,3 @@ def unfold(proc, *inits):
 #    def identity(*args):  # unpythonic.fun.identity, but dependency loop
 #        return args
 #    return mapr(identity, *iterables)
-
-def test():
-    from operator import add, mul
-    from unpythonic.fun import curry, composer, composerc, composel, to1st, rotate
-    from unpythonic.llist import cons, nil, ll, lreverse
-    from unpythonic.it import take, tail
-
-    # scan/accumulate: lazy fold that yields intermediate results.
-    assert tuple(scanl(add, 0, range(1, 5))) == (0, 1, 3, 6, 10)
-    assert tuple(scanr(add, 0, range(1, 5))) == (10, 9, 7, 4, 0)
-    assert tuple(scanl(mul, 1, range(2, 6))) == (1, 2, 6, 24, 120)
-    assert tuple(scanr(mul, 1, range(2, 6))) == (120, 60, 20, 5, 1)
-
-    assert tuple(scanl(cons, nil, ll(1, 2, 3))) == (nil, ll(1), ll(2, 1), ll(3, 2, 1))
-    assert tuple(scanr(cons, nil, ll(1, 2, 3))) == (ll(1, 2, 3), ll(2, 3), ll(3), nil)
-
-    # in contrast, fold just returns the final result.
-    assert foldl(cons, nil, ll(1, 2, 3)) == ll(3, 2, 1)
-    assert foldr(cons, nil, ll(1, 2, 3)) == ll(1, 2, 3)
-
-    # reduce is a fold with a single input, with init optional.
-    assert reducel(add, (1, 2, 3)) == 6
-    assert reducer(add, (1, 2, 3)) == 6
-
-    # scanl1, scanr1 are a scan with a single input, with init optional.
-    assert tuple(scanl1(add, (1, 2, 3))) == (1, 3, 6)
-    assert tuple(scanr1(add, (1, 2, 3))) == (6, 5, 3)
-
-    psums = composer(tail, curry(scanl, add, 0))  # tail to drop the init value
-    pprods = composer(tail, curry(scanl, mul, 1))
-    data = range(1, 5)
-    assert tuple(psums(data)) == (1, 3, 6, 10)
-    assert tuple(pprods(data)) == (1, 2, 6, 24)
-
-    psums = curry(scanl1, add)  # or use the fact the 1-input variant needs no init
-    pprods = curry(scanl1, mul)
-    data = range(1, 5)
-    assert tuple(psums(data)) == (1, 3, 6, 10)
-    assert tuple(pprods(data)) == (1, 2, 6, 24)
-
-    def append_tuple(a, b, acc):
-        return acc + ((a, b),)
-    assert foldl(append_tuple, (), (1, 2, 3), (4, 5)) == ((1, 4), (2, 5))
-    assert foldr(append_tuple, (), (1, 2, 3), (4, 5)) == ((2, 5), (1, 4))
-    assert rfoldl(append_tuple, (), (1, 2, 3), (4, 5)) == ((3, 5), (2, 4))
-    assert tuple(rscanl(append_tuple, (), (1, 2, 3), (4, 5))) == ((), ((3, 5),), ((3, 5), (2, 4)))
-
-    def mymap_one(f, iterable):
-        f_then_cons = composer(cons, to1st(f))  # args: elt, acc
-        return foldr(f_then_cons, nil, iterable)
-    double = lambda x: 2 * x
-    assert mymap_one(double, ll(1, 2, 3)) == ll(2, 4, 6)
-    def mymap_one2(f, iterable):
-        f_then_cons = composel(to1st(f), cons)  # args: elt, acc
-        return foldr(f_then_cons, nil, iterable)
-    assert mymap_one2(double, ll(1, 2, 3)) == ll(2, 4, 6)
-
-    # point-free-ish style
-    mymap_one3 = lambda f: partial(foldr, composer(cons, to1st(f)), nil)
-    doubler = mymap_one3(double)
-    assert doubler(ll(1, 2, 3)) == ll(2, 4, 6)
-
-    try:
-        doubler(ll(1, 2, 3), ll(4, 5, 6))
-    except TypeError:
-        pass
-    else:
-        assert False  # one arg too many; cons in the compose chain expects 2 args (acc is one)
-
-    # minimum arity of fold functions is 3, to allow use with curry:
-    mymap_one4 = lambda f: curry(foldr, composer(cons, to1st(f)), nil)
-    doubler = mymap_one4(double)
-    assert doubler(ll(1, 2, 3)) == ll(2, 4, 6)
-
-    # curry supports passing through on the right any args over the max arity.
-    # If an intermediate result is a callable, it is invoked on the remaining
-    # positional args:
-    assert curry(mymap_one4, double, ll(1, 2, 3)) == ll(2, 4, 6)
-
-    # But having any args remaining when the top-level curry context exits
-    # is an error:
-    try:
-        curry(double, 2, "foo")
-    except TypeError:
-        pass
-    else:
-        assert False   # top-level curry context exited with args remaining
-
-    # This also works; curried f takes one argument and the second one is passed
-    # through on the right; this two-tuple then ends up as the arguments to cons.
-    mymap_one5 = lambda f: curry(foldr, composer(cons, curry(f)), nil)
-    assert curry(mymap_one5, double, ll(1, 2, 3)) == ll(2, 4, 6)
-
-    # Finally, we can drop the inner curry by using a currying compose.
-    # This is as close to "(define (map f) (foldr (compose cons f) empty)"
-    # (#lang spicy) as we're gonna get in Python.
-    mymap = lambda f: curry(foldr, composerc(cons, f), nil)
-    assert curry(mymap, double, ll(1, 2, 3)) == ll(2, 4, 6)
-
-    # The currying has actually made it not just map one, but general map that
-    # accepts multiple inputs.
-    #
-    # The iterables are taken by the processing function. acc, being the last
-    # argument, is passed through on the right. The output from the processing
-    # function - one new item - and acc then become a two-tuple, which gets
-    # passed into cons.
-    myadd = lambda x, y: x + y  # can't inspect signature of builtin add
-    assert curry(mymap, myadd, ll(1, 2, 3), ll(2, 4, 6)) == ll(3, 6, 9)
-
-    # map_longest. foldr would walk the inputs from the right; use foldl.
-    mymap_longestrev = lambda f: curry(foldl, composerc(cons, f), nil, longest=True)
-    mymap_longest = composerc(lreverse, mymap_longestrev)
-    def noneadd(a, b):
-        if all(x is not None for x in (a, b)):
-            return a + b
-    assert curry(mymap_longest, noneadd, ll(1, 2, 3), ll(2, 4)) == ll(3, 6, None)
-
-    reverse_one = curry(foldl, cons, nil)
-    assert reverse_one(ll(1, 2, 3)) == ll(3, 2, 1)
-
-    append_two = lambda a, b: foldr(cons, b, a)  # a, b: linked lists
-    assert append_two(ll(1, 2, 3), ll(4, 5, 6)) == ll(1, 2, 3, 4, 5, 6)
-
-    # see upythonic.llist.lappend
-    append_many = lambda *lsts: foldr(append_two, nil, lsts)
-    assert append_many(ll(1, 2), ll(3, 4), ll(5, 6)) == ll(1, 2, 3, 4, 5, 6)
-
-    mysum = curry(foldl, add, 0)
-    myprod = curry(foldl, mul, 1)
-    a = ll(1, 2)
-    b = ll(3, 4)
-    assert mysum(append_two(a, b)) == 10
-    assert myprod(b) == 12
-
-    packtwo = lambda a, b: ll(a, b)  # using a tuple return value here would confuse curry.
-    assert foldl(composerc(cons, packtwo), nil, (1, 2, 3), (4, 5), longest=True) == \
-           ll(ll(3, None), ll(2, 5), ll(1, 4))
-
-    @rotate(-1)
-    def zipper(acc, *rest):   # so that we can use the *args syntax to declare this
-        return acc + (rest,)  # even though the input is (e1, ..., en, acc).
-#    def zipper(*args):  # straightforward version
-#        *rest, acc = args
-#        return acc + (tuple(rest),)
-    zipl1 = curry(foldl, zipper, ())
-    zipr1 = curry(foldr, zipper, ())
-    assert zipl1((1, 2, 3), (4, 5, 6), (7, 8)) == ((1, 4, 7), (2, 5, 8))
-    assert zipr1((1, 2, 3), (4, 5, 6), (7, 8)) == ((2, 5, 8), (1, 4, 7))
-
-    # Unfold.
-    #
-    def step2(k):  # x0, x0 + 2, x0 + 4, ...
-        return (k, k + 2)
-
-    def fibo(a, b):
-        return (a, b, a + b)
-
-    def myiterate(f, x):  # x0, f(x0), f(f(x0)), ...
-        return (x, f, f(x))
-
-    def zip_two(As, Bs):
-        if len(As) and len(Bs):
-            (A0, *moreAs), (B0, *moreBs) = As, Bs
-            return ((A0, B0), moreAs, moreBs)
-
-    assert tuple(take(10, unfold1(step2, 10))) == (10, 12, 14, 16, 18, 20, 22, 24, 26, 28)
-    assert tuple(take(10, unfold(fibo, 1, 1))) == (1, 1, 2, 3, 5, 8, 13, 21, 34, 55)
-    assert tuple(take(5, unfold(myiterate, lambda x: x**2, 2))) == (2, 4, 16, 256, 65536)
-    assert tuple(unfold(zip_two, (1, 2, 3, 4), (5, 6, 7))) == ((1, 5), (2, 6), (3, 7))
-
-    print("All tests PASSED")
-
-if __name__ == '__main__':
-    test()
