@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """Missing batteries for functools.
 
@@ -14,15 +13,18 @@ __all__ = ["memoize", "curry", "iscurried",
            "composer1", "composel1", "composer1i", "composel1i",  # single arg
            "composer",  "composel",  "composeri",  "composeli",   # multi-arg
            "composerc", "composelc", "composerci", "composelci",  # multi-arg w/ curry
-           "to1st", "to2nd", "tokth", "tolast", "to"]
+           "to1st", "to2nd", "tokth", "tolast", "to",
+           "withself"]
 
 from functools import wraps, partial
 from operator import itemgetter
 
-from unpythonic.arity import arities, UnknownArity
-from unpythonic.fold import reducel
-from unpythonic.dynscope import dyn, make_dynvar
+from .arity import arities, UnknownArity
+from .fold import reducel
+from .dynassign import dyn, make_dynvar
+from .regutil import register_decorator
 
+@register_decorator(priority=10)
 def memoize(f):
     """Decorator: memoize the function f.
 
@@ -62,8 +64,7 @@ def memoize(f):
 #        return memo[k]
 #    return memoized
 
-make_dynvar(curry_toplevel_passthrough=False)
-make_dynvar(_curry_context=[])
+make_dynvar(curry_context=[])
 make_dynvar(_curry_allow_uninspectable=False)  # if True, no-op if not inspectable.
 def _currycall(f, *args, **kwargs):
     """Co-operate with unpythonic.syntax.curry.
@@ -74,6 +75,8 @@ def _currycall(f, *args, **kwargs):
     Hence we provide this separate mode to curry-and-call even if no args.
     """
     return curry(f, *args, _curry_force_call=True, **kwargs)
+
+@register_decorator(priority=90)
 def curry(f, *args, _curry_force_call=False, **kwargs):
     """Decorator: curry the function f.
 
@@ -132,9 +135,11 @@ def curry(f, *args, _curry_force_call=False, **kwargs):
     because such usage often indicates a bug.
 
     This behavior can be locally modified by setting the dynvar
-    ``curry_toplevel_passthrough``::
+    ``curry_context``, which is a list representing the stack of
+    currently active curry contexts. A context is any object,
+    a human-readable label is fine::
 
-        with dyn.let(curry_toplevel_passthrough=True):
+        with dyn.let(curry_context=["whatever"]):
             curry(double, 2, "foo") == (4, "foo")
 
     Because it is a dynvar, it affects all ``curry`` calls in its dynamic extent,
@@ -179,8 +184,8 @@ def curry(f, *args, _curry_force_call=False, **kwargs):
         return f
     @wraps(f)
     def curried(*args, **kwargs):
-        outerctx = dyn._curry_context
-        with dyn.let(_curry_context=(outerctx + [f])):
+        outerctx = dyn.curry_context
+        with dyn.let(curry_context=(outerctx + [f])):
             if len(args) < min_arity:
                 return curry(partial(f, *args, **kwargs))
             # passthrough on right, like https://github.com/Technologicat/spicy
@@ -193,7 +198,7 @@ def curry(f, *args, _curry_force_call=False, **kwargs):
                     if not iscurried(now_result):
                         now_result = curry(now_result)
                     return now_result(*later_args)
-                if not (outerctx or dyn.curry_toplevel_passthrough):
+                if not outerctx:
                     raise TypeError("Top-level curry context exited with {:d} arg(s) remaining: {}".format(len(later_args),
                                                                                                            later_args))
                 # pass through to the curried procedure waiting in outerctx
@@ -433,7 +438,7 @@ def _make_compose(direction):  # "left", "right"
                 # co-operate with curry: provide a top-level curry context
                 # to allow passthrough from the function that is applied first
                 # to the function that is applied second.
-                bindings = {"_curry_context": dyn._curry_context + [composed]}
+                bindings = {"curry_context": dyn.curry_context + [composed]}
             with dyn.let(**bindings):
                 a = g(*args)
             # we could duck-test, but this is more predictable for the user
@@ -569,153 +574,32 @@ def to(*specs):
     """
     return composeli(tokth(k, f) for k, f in specs)
 
-def test():
-    from collections import Counter
-    evaluations = Counter()
-    @memoize
-    def f(x):
-        evaluations[x] += 1
-        return x**2
-    f(3)
-    f(3)
-    f(4)
-    f(3)
-    assert all(n == 1 for n in evaluations.values())
+@register_decorator(priority=80)
+def withself(f):
+    """Decorator. Allow a lambda to refer to itself.
 
-    # "memoize lambda": classic evaluate-at-most-once thunk
-    thunk = memoize(lambda: print("hi from thunk"))
-    thunk()
-    thunk()
+    This is essentially the Y combinator trick packaged as a decorator.
 
-    evaluations = 0
-    @memoize
-    def t():
-        nonlocal evaluations
-        evaluations += 1
-    t()
-    t()
-    assert evaluations == 1
+    The reference to the lambda itself (the ``self`` argument) is passed as the
+    first positional argument. It is declared explicitly, but passed implicitly,
+    just like the ``self`` argument of a method.
 
-    # exception storage in memoize
-    class AllOkJustTesting(Exception):
-        pass
-    evaluations = 0
-    @memoize
-    def t():
-        nonlocal evaluations
-        evaluations += 1
-        raise AllOkJustTesting()
-    olderr = None
-    for _ in range(3):
-        try:
-            t()
-        except AllOkJustTesting as err:
-            if olderr is not None and err is not olderr:
-                assert False  # exception instance memoized, should be same every time
-            olderr = err
-        else:
-            assert False  # memoize should not block raise
-    assert evaluations == 1
+    Note there is no point using this with named functions, because they can
+    already refer to themselves via the name.
 
-    @curry
-    def add3(a, b, c):
-        return a + b + c
-    assert add3(1)(2)(3) == 6
-    # actually uses partial application so these work, too
-    assert add3(1, 2)(3) == 6
-    assert add3(1)(2, 3) == 6
-    assert add3(1, 2, 3) == 6
+    Example::
 
-    @curry
-    def lispyadd(*args):
-        return sum(args)
-    assert lispyadd() == 0  # no args is a valid arity here
+        fact = withself(lambda self, n: n * self(n - 1) if n > 1 else 1)
+        assert fact(5) == 120
 
-    @curry
-    def foo(a, b, *, c, d):
-        return a, b, c, d
-    assert foo(5, c=23)(17, d=42) == (5, 17, 23, 42)
+    To TCO it, too::
 
-    # currying a thunk is essentially a no-op
-    evaluations = 0
-    @curry
-    def t():
-        nonlocal evaluations
-        evaluations += 1
-    t()
-    assert evaluations == 1  # t has no args, so it should have been invoked
-
-    add = lambda x, y: x + y
-    a = curry(add)
-    assert curry(a) is a  # curry wrappers should not stack
-
-    # flip
-    def f(a, b):
-        return (a, b)
-    assert f(1, 2) == (1, 2)
-    assert (flip(f))(1, 2) == (2, 1)
-    assert (flip(f))(1, b=2) == (1, 2)  # b -> kwargs
-
-    double = lambda x: 2*x
-    inc    = lambda x: x+1
-    inc_then_double = composer1(double, inc)
-    double_then_inc = composel1(double, inc)
-    assert inc_then_double(3) == 8
-    assert double_then_inc(3) == 7
-
-    inc2_then_double = composer1(double, inc, inc)
-    double_then_inc2 = composel1(double, inc, inc)
-    assert inc2_then_double(3) == 10
-    assert double_then_inc2(3) == 8
-
-    inc_then_double = composer(double, inc)
-    double_then_inc = composel(double, inc)
-    assert inc_then_double(3) == 8
-    assert double_then_inc(3) == 7
-
-    inc2_then_double = composer(double, inc, inc)
-    double_then_inc2 = composel(double, inc, inc)
-    assert inc2_then_double(3) == 10
-    assert double_then_inc2(3) == 8
-
-    assert to1st(double)(1, 2, 3)  == (2, 2, 3)
-    assert to2nd(double)(1, 2, 3)  == (1, 4, 3)
-    assert tolast(double)(1, 2, 3) == (1, 2, 6)
-
-    processor = to((0, double),
-                   (-1, inc),
-                   (1, composer(double, double)),
-                   (0, inc))
-    assert processor(1, 2, 3) == (3, 8, 4)
-
-    assert identity(1, 2, 3) == (1, 2, 3)
-    assert (rotate(-1)(identity))(1, 2, 3) == (3, 1, 2)
-    assert (rotate(1)(identity))(1, 2, 3) == (2, 3, 1)
-
-    # inner to outer: (a, b, c) -> (b, c, a) -> (a, c, b)
-    assert flip(rotate(-1)(identity))(1, 2, 3) == (1, 3, 2)
-
-    def hello(*args):
-        return args
-    assert apply(hello, (1, 2, 3)) == (1, 2, 3)
-    assert apply(hello, 1, (2, 3, 4)) == (1, 2, 3, 4)
-    assert apply(hello, 1, 2, (3, 4, 5)) == (1, 2, 3, 4, 5)
-    assert apply(hello, 1, 2, [3, 4, 5]) == (1, 2, 3, 4, 5)
-
-    assert const(1, 2, 3)(42, "foo") == (1, 2, 3)
-    assert notf(lambda x: 2*x)(3) is False
-    assert notf(lambda x: 2*x)(0) is True
-    isint  = lambda x: isinstance(x, int)
-    iseven = lambda x: x % 2 == 0
-    isstr  = lambda s: isinstance(s, str)
-    assert andf(isint, iseven)(42) is True
-    assert andf(isint, iseven)(43) is False
-    pred = orf(isstr, andf(isint, iseven))
-    assert pred(42) is True
-    assert pred("foo") is True
-    assert pred(None) is False  # neither condition holds
-
-    print("All tests PASSED")
-
-if __name__ == '__main__':
-    test()
+        fact = trampolined(withself(lambda self, n, acc=1:
+                             acc if n == 0 else jump(self, n - 1, n * acc)))
+        assert fact(5) == 120
+        fact(5000)  # no crash
+    """
+    @wraps(f)
+    def fwithself(*args, **kwargs):
+        return f(fwithself, *args, **kwargs)
+    return fwithself
