@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """Missing batteries for itertools.
 
@@ -11,20 +10,49 @@ and (stream-scan) in SRFI-41.
 
 ``flatten`` based on Danny Yoo's version:
   http://rightfootin.blogspot.fi/2006/09/more-on-python-flatten.html
+
+``uniqify``, ``uniq``,  ``take``, ``drop``, ``partition`` just package
+``itertools`` recipes.
 """
 
-__all__ = ["map_longest", "mapr", "zipr", "mapr_longest", "zipr_longest",
-           "flatmap", "uniqify", "uniq",
-           "take", "drop", "split_at", "unpack",
-           "tail", "first", "second", "nth", "last", "scons",
+__all__ = ["rev", "map_longest",
+           "rmap", "rzip", "rmap_longest", "rzip_longest",
+           "mapr", "zipr", "mapr_longest", "zipr_longest",
+           "flatmap",
+           "uniqify", "uniq",
+           "take", "drop", "split_at",
+           "unpack",
+           "tail", "butlast", "butlastn",
+           "first", "second", "nth", "last",
+           "scons",
            "flatten", "flatten1", "flatten_in",
-           "iterate", "iterate1"]
+           "iterate", "iterate1",
+           "partition"]
 
 from operator import itemgetter
-from functools import partial
 from itertools import tee, islice, zip_longest, starmap, chain, filterfalse, groupby
 from collections import deque
 
+def rev(iterable):
+    """Reverse an iterable.
+
+    If a sequence, the return value is ``reversed(iterable)``.
+
+    Otherwise the return value is ``reversed(tuple(iterable))``.
+
+    Hence generators will be fully evaluated until they stop; the input
+    ``iterable`` must be finite for ``rev`` to make any sense.
+    """
+    # Unlike further below, here we "return" instead of "yield from",
+    # because "rev" is such a thin layer of abstraction that it has become
+    # effectively transparent (PG, "On Lisp"). The call site expects
+    # reversed output, and the "reversed" generator is the standard
+    # pythonic representation for that.
+    try:  # maybe a sequence?
+        return reversed(iterable)
+    except TypeError:
+        return reversed(tuple(iterable))
+
 # When completing an existing set of functions (map, zip, zip_longest),
 # consistency wins over curry-friendliness.
 def map_longest(func, *iterables, fillvalue=None):
@@ -33,23 +61,111 @@ def map_longest(func, *iterables, fillvalue=None):
     In the input to ``func``, missing elements (after end of shorter inputs)
     are replaced by ``fillvalue``, which defaults to ``None``.
     """
-    return starmap(func, zip_longest(*iterables, fillvalue=fillvalue))
+    # "yield from" semantically better here than "return", because the call site
+    # sees a "map_longest" generator object instead of a "starmap" generator
+    # object. This describes explicitly what the generator does, and is in line
+    # with the terminology used at the call site.
+    yield from starmap(func, zip_longest(*iterables, fillvalue=fillvalue))
 
-def mapr(func, *sequences):
-    """Like map, but walk each sequence from the right."""
-    return map(func, *(reversed(s) for s in sequences))
+def rmap(func, *iterables):
+    """Like map, but from the right.
 
-def zipr(*sequences):
-    """Like zip, but walk each sequence from the right."""
-    return zip(*(reversed(s) for s in sequences))
+    For multiple inputs with different lengths, ``rmap`` syncs the **right** ends.
+    See ``mapr`` for the variant that syncs the **left** ends.
 
-def mapr_longest(func, *sequences, fillvalue=None):
-    """Like map_longest, but walk each sequence from the right."""
-    return map_longest(func, *(reversed(s) for s in sequences), fillvalue=fillvalue)
+    ``rev`` is applied to the inputs. Note this forces any generators.
 
-def zipr_longest(*sequences, fillvalue=None):
-    """Like itertools.zip_longest, but walk each sequence from the right."""
-    return zip_longest(*(reversed(s) for s in sequences), fillvalue=fillvalue)
+    Examples::
+
+        from operator import add
+
+        # just map, for comparison:
+        assert tuple(map(add, (1, 2, 3), (4, 5))) == (5, 7)
+
+        # reverse each, then map; syncs right ends:
+        # rmap(f, ...) = map(f, rev(s) for s in ...)
+        assert tuple(rmap(add, (1, 2, 3), (4, 5))) == (8, 6)
+
+        # map, then reverse; syncs left ends:
+        # mapr(f, ...) = rev(map(f, ...))
+        assert tuple(mapr(add, (1, 2, 3), (4, 5))) == (7, 5)
+    """
+    yield from map(func, *(rev(s) for s in iterables))
+
+def rzip(*iterables):
+    """Like zip, but from the right.
+
+    For multiple inputs with different lengths, ``rzip`` syncs the **right** ends.
+    See ``zipr`` for the variant that syncs the **left** ends.
+
+    ``rev`` is applied to the inputs. Note this forces any generators.
+
+    Examples::
+
+        # just zip, for comparison:
+        assert tuple(zip((1, 2, 3), (4, 5))) == ((1, 4), (2, 5))
+
+        # reverse each, then zip; syncs right ends:
+        # rzip(...) = zip(rev(s) for s in ...)
+        assert tuple(rzip((1, 2, 3), (4, 5))) == ((3, 5), (2, 4))
+
+        # zip, then reverse; syncs left ends:
+        # zipr(...) = rev(zip(...))
+        assert tuple(zipr((1, 2, 3), (4, 5))) == ((2, 5), (1, 4))
+    """
+    yield from zip(*(rev(s) for s in iterables))
+
+def rmap_longest(func, *iterables, fillvalue=None):
+    """Like rmap, but terminate on the longest input."""
+    yield from map_longest(func, *(rev(s) for s in iterables), fillvalue=fillvalue)
+
+def rzip_longest(*iterables, fillvalue=None):
+    """Like rzip, but terminate on the longest input."""
+    yield from zip_longest(*(rev(s) for s in iterables), fillvalue=fillvalue)
+
+def mapr(proc, *iterables):
+    """Like map, but from the right.
+
+    For multiple inputs with different lengths, ``mapr`` syncs the **left** ends.
+    See ``rmap`` for the variant that syncs the **right** ends.
+    """
+    yield from rev(map(proc, *iterables))
+
+def zipr(*iterables):
+    """Like zip, but from the right.
+
+    For multiple inputs with different lengths, ``zipr`` syncs the **left** ends.
+    See ``rzip`` for the variant that syncs the **right** ends.
+    """
+    yield from rev(zip(*iterables))
+
+def mapr_longest(proc, *iterables, fillvalue=None):
+    """Like mapr, but terminate on the longest input."""
+    yield from rev(map_longest(proc, *iterables, fillvalue=fillvalue))
+
+def zipr_longest(*iterables, fillvalue=None):
+    """Like zipr, but terminate on the longest input."""
+    yield from rev(zip_longest(*iterables, fillvalue=fillvalue))
+
+# Equivalent recursive process:
+#def _mapr(proc, iterable0, *iterables, longest=False, fillvalue=None):
+#    z = zip if not longest else partial(zip_longest, fillvalue=fillvalue)
+#    xss = z(iterable0, *iterables)
+#    def _mapr_recurser():
+#        try:
+#            xs = next(xss)
+#        except StopIteration:
+#            return
+#        subgen = _mapr_recurser()
+#        yield from subgen
+#        yield proc(*xs)
+#    return _mapr_recurser()
+#
+#def _zipr(iterable0, *iterables, longest=False, fillvalue=None):
+#    def identity(*args):  # unpythonic.fun.identity, but dependency loop
+#        return args
+#    return _mapr(identity, iterable0, *iterables,
+#                 longest=longest, fillvalue=fillvalue)
 
 def flatmap(f, iterable0, *iterables):
     """Map, then concatenate results.
@@ -82,7 +198,7 @@ def flatmap(f, iterable0, *iterables):
         assert tuple(flatmap(sum_and_diff, (10, 20, 30), (1, 2, 3))) == \\
                (11, 9, 22, 18, 33, 27)
     """
-    return chain.from_iterable(map(f, iterable0, *iterables))
+    yield from chain.from_iterable(map(f, iterable0, *iterables))
 #    for xs in map(f, iterable0, *iterables):
 #        yield from xs
 
@@ -119,7 +235,7 @@ def uniq(iterable, *, key=None):
     This is ``unique_justseen`` from ``itertools`` recipes.
     """
     # the outer map retrieves the item from the subiterator in (key, subiterator).
-    return map(next, map(itemgetter(1), groupby(iterable, key)))
+    yield from map(next, map(itemgetter(1), groupby(iterable, key)))
 
 def take(n, iterable):
     """Return an iterator that yields the first n items of iterable, then stops.
@@ -179,8 +295,7 @@ def split_at(n, iterable):
 def unpack(n, iterable, *, k=None, fillvalue=None):
     """From iterable, return the first n elements, and the kth tail.
 
-    This is a lazy generalization of sequence unpacking that works also for
-    infinite iterables.
+    Lazy generalization of sequence unpacking, works also for infinite iterables.
 
     Default ``k=None`` means ``k = n``, i.e. return the tail that begins
     right after the extracted items. Other values are occasionally useful,
@@ -223,13 +338,13 @@ def unpack(n, iterable, *, k=None, fillvalue=None):
             if j == k:  # tail is desired to overlap with the extracted items
                 it, tl = tee(it)
             out.append(next(it))
-        except StopIteration:  # fewer than n items
+        except StopIteration:  # had fewer than n items remaining
             out += [fillvalue] * (n - len(out))
-            def empty_sequence():
+            def empty_iterable():
                 yield from ()
-            tl = empty_sequence()
+            tl = empty_iterable()
             break
-    if not tl:  # avoid replacing empty_sequence()
+    if not tl:  # avoid replacing empty_iterable()
         if k == n:
             tl = it
         elif k > n:
@@ -244,6 +359,33 @@ def tail(iterable):
     """
     return drop(1, iterable)
 
+def butlast(iterable):
+    """Yield all items from iterable, except the last one (if iterable is finite).
+
+    Return a generator.
+
+    Uses intermediate storage - do not use the original iterator after calling
+    ``butlast``.
+    """
+    return butlastn(1, iterable)
+
+def butlastn(n, iterable):
+    """Yield all items from iterable, except the last n (if iterable is finite).
+
+    Return a generator.
+
+    Uses intermediate storage - do not use the original iterator after calling
+    ``butlastn``.
+    """
+    # we let StopIteration propagate from anything that could raise it here.
+    it = iter(iterable)
+    q = deque()
+    for _ in range(n+1):
+        q.append(next(it))
+    while True:
+        yield q.popleft()
+        q.append(next(it))
+
 def first(iterable, *, default=None):
     """Like nth, but return the first item."""
     return nth(0, iterable, default=default)
@@ -261,7 +403,7 @@ def nth(n, iterable, *, default=None):
         raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
-    it = drop(n - 1, iterable) if n else iter(iterable)
+    it = drop(n, iterable) if n else iter(iterable)
     try:
         return next(it)
     except StopIteration:
@@ -305,7 +447,7 @@ def flatten(iterable, pred=None):
 
     E.g. to flatten only those items that contain only tuples::
 
-        is_nested = lambda e: all(isinstance(x, (tuple, list)) for x in e)
+        is_nested = lambda e: all(isinstance(x, (list, tuple)) for x in e)
         data = (((1, 2), (3, 4)), (5, 6))
         assert tuple(flatten(data, is_nested)) == ((1, 2), (3, 4), (5, 6))
     """
@@ -313,10 +455,9 @@ def flatten(iterable, pred=None):
 
 def flatten1(iterable, pred=None):
     """Like flatten, but process outermost level only."""
-    if pred:
-        return _flatten(iterable, pred, recursive=False)
-    else:
+    if not pred:
         return chain.from_iterable(iterable)  # itertools recipes: fast, no pred
+    return _flatten(iterable, pred, recursive=False)
 
 def _flatten(iterable, pred=None, recursive=True):
     pred = pred or (lambda x: True)  # unpythonic.fun.const(True), but dependency loop
@@ -336,7 +477,7 @@ def flatten_in(iterable, pred=None):
 
     Example::
 
-        is_nested = lambda e: all(isinstance(x, (tuple, list)) for x in e)
+        is_nested = lambda e: all(isinstance(x, (list, tuple)) for x in e)
         data = (((1, 2), ((3, 4), (5, 6)), 7), ((8, 9), (10, 11)))
         assert tuple(flatten(data, is_nested))    == \\
                (((1, 2), ((3, 4), (5, 6)), 7), (8, 9), (10, 11))
@@ -375,135 +516,24 @@ def iterate(f, *args):
         yield args
         args = f(*args)
 
-def test():
-    from operator import add
-    from unpythonic.fun import composel, identity
+def partition(pred, iterable):
+    """Partition an iterable to entries satifying and not satisfying a predicate.
 
-    def noneadd(a, b):
-        if all(x is not None for x in (a, b)):
-            return a + b
+    Return two generators, ``(false-items, true-items)``, where each generator
+    yields those items from ``iterable`` for which ``pred`` gives the indicated value.
 
-    # Adding the missing batteries to the algebra of map and zip.
-    # Note Python's (and Racket's) map is like Haskell's zipWith, but for n inputs.
-    assert tuple(map(add, (1, 2), (3, 4))) == (4, 6)  # builtin
-    assert tuple(mapr(add, (1, 2), (3, 4))) == (6, 4)
-    assert tuple(zip((1, 2, 3), (4, 5, 6), (7, 8))) == ((1, 4, 7), (2, 5, 8))  # builtin
-    assert tuple(zipr((1, 2, 3), (4, 5, 6), (7, 8))) == ((3, 6, 8), (2, 5, 7))
-    assert tuple(map_longest(noneadd, (1, 2, 3), (2, 4))) == (3, 6, None)
-    assert tuple(mapr_longest(noneadd, (1, 2, 3), (2, 4))) == (7, 4, None)
-    assert tuple(zip_longest((1, 2, 3), (2, 4))) == ((1, 2), (2, 4), (3, None))  # itertools
-    assert tuple(zipr_longest((1, 2, 3), (2, 4))) == ((3, 4), (2, 2), (1, None))
+    This is ``partition`` from ``itertools`` recipes.
 
-    assert tuple(scons(0, range(1, 5))) == tuple(range(5))
-    assert tuple(tail(scons("foo", range(5)))) == tuple(range(5))
+    **Caution**: infinite inputs require some care in order not to cause a blowup
+    in the amount of intermediate storage needed. The original iterable is walked
+    only once (because that's all we can generally do!), and depending on the
+    content of ``iterable`` and in which order the outputs are read, an indefinite
+    number of either false-items or true-items may build up in the intermediate storage.
 
-    def msqrt(x):  # multivalued sqrt
-        if x == 0.:
-            return (0.,)
-        else:
-            s = x**0.5
-            return (s, -s)
-    assert tuple(flatmap(msqrt, (0, 1, 4, 9))) == (0., 1., -1., 2., -2., 3., -3.)
-
-    def add_and_tuplify(a, b):
-        return (a + b,)
-    assert tuple(flatmap(add_and_tuplify, (10, 20, 30), (1, 2, 3))) == (11, 22, 33)
-
-    def sum_and_diff(a, b):
-        return (a + b, a - b)
-    assert tuple(flatmap(sum_and_diff, (10, 20, 30), (1, 2, 3))) == (11, 9, 22, 18, 33, 27)
-
-    assert tuple(take(10, range(100))) == tuple(range(10))
-    assert tuple(take(10, range(3))) == tuple(range(3))
-
-    assert tuple(drop(5, range(10))) == tuple(range(5, 10))
-    assert tuple(drop(5, range(3))) == ()
-
-    drop5take5 = composel(partial(drop, 5), partial(take, 5))
-    assert tuple(drop5take5(range(20))) == tuple(range(5, 10))
-
-    with_same_n = lambda n, fs: (partial(f, n) for f in fs)
-#    with_same_n = lambda n, fs: map((lambda f: partial(f, n)), fs)
-    drop5take5 = composel(*with_same_n(5, (drop, take)))
-    assert tuple(drop5take5(range(20))) == tuple(range(5, 10))
-
-    with_n = lambda *args: (partial(f, n) for n, f in args)
-    drop5take10 = composel(*with_n((5, drop), (10, take)))
-    assert tuple(drop5take10(range(20))) == tuple(range(5, 15))
-
-    a, b = map(tuple, split_at(5, range(10)))
-    assert a == tuple(range(5))
-    assert b == tuple(range(5, 10))
-
-    a, b = map(tuple, split_at(5, range(3)))
-    assert a == tuple(range(3))
-    assert b == ()
-
-    # Python's builtin map is not curry-friendly; it accepts arity 1,
-    # but actually requires 2. Solution: use partial instead of curry.
-    zipl2 = partial(map, identity)
-    zipr2 = lambda *sequences: map(identity, *(reversed(s) for s in sequences))
-    assert tuple(zipl2((1, 2, 3), (4, 5, 6), (7, 8))) == ((1, 4, 7), (2, 5, 8))
-    assert tuple(zipr2((1, 2, 3), (4, 5, 6), (7, 8))) == ((3, 6, 8), (2, 5, 7))
-
-    zipr3 = partial(mapr, identity)
-    assert tuple(zipr3((1, 2, 3), (4, 5, 6), (7, 8))) == ((3, 6, 8), (2, 5, 7))
-
-    assert tuple(uniqify((1, 1, 2, 2, 2, 2, 4, 3, 3, 3))) == (1, 2, 4, 3)
-    data = (('foo', 1), ('bar', 1), ('foo', 2), ('baz', 2), ('qux', 4), ('foo', 3))
-    assert tuple(uniqify(data, key=itemgetter(0))) == (('foo', 1), ('bar', 1), ('baz', 2), ('qux', 4))
-    assert tuple(uniqify(data, key=itemgetter(1))) == (('foo', 1), ('foo', 2), ('qux', 4), ('foo', 3))
-
-    assert tuple(uniq((1, 1, 2, 2, 2, 1, 2, 2, 4, 3, 4, 3, 3))) == (1, 2, 1, 2, 4, 3, 4, 3)
-
-    assert tuple(flatten(((1, 2), (3, (4, 5), 6), (7, 8, 9)))) == (1, 2, 3, 4, 5, 6, 7, 8, 9)
-    assert tuple(flatten1(((1, 2), (3, (4, 5), 6), (7, 8, 9)))) == (1, 2, 3, (4, 5), 6, 7, 8, 9)
-
-    is_nested = lambda e: all(isinstance(x, (tuple, list)) for x in e)
-    assert tuple(flatten((((1, 2), (3, 4)), (5, 6)), is_nested)) == ((1, 2), (3, 4), (5, 6))
-
-    data = (((1, 2), ((3, 4), (5, 6)), 7), ((8, 9), (10, 11)))
-    assert tuple(flatten(data, is_nested))    == (((1, 2), ((3, 4), (5, 6)), 7), (8, 9), (10, 11))
-    assert tuple(flatten_in(data, is_nested)) == (((1, 2), (3, 4), (5, 6), 7),   (8, 9), (10, 11))
-
-    # lazy unpack from a sequence
-    a, b, c, tl = unpack(3, range(5))
-    assert a == 0 and b == 1 and c == 2
-    assert next(tl) == 3
-
-    # lazy unpack falling off the end of a sequence
-    a, b, c, tl = unpack(3, range(2))
-    assert a == 0 and b == 1 and c is None
-    try:
-        next(tl)
-    except StopIteration:
-        pass
-    else:
-        assert False  # the tail should be empty
-
-    # unpacking of generators - careful!
-    def mygen():
-        for x in range(10, 16):
-            yield x
-    def dostuff(s):
-        a, b, tl = unpack(2, s, k=0)  # peek two, set tl to 0th tail (the original s)
-        return tl
-    g = mygen()
-    dostuff(g)  # gotcha: advances g!
-    assert next(g) == 12
-
-    # workaround 1: tee off a copy with itertools.tee:
-    g = mygen()
-    g, h = tee(g)
-    dostuff(g)  # advances g, but not h
-    assert next(h) == 10
-
-    # workaround 2: use the implicit tee in unpack:
-    g = mygen()
-    g = dostuff(g)  # advances g, but then overwrites name g with the returned tail
-    assert next(g) == 10
-
-    print("All tests PASSED")
-
-if __name__ == '__main__':
-    test()
+    (Example: partition the natural numbers, and only ever read the even numbers.
+    It will eventually run out of memory storing all the odd numbers "to be read
+    later".)
+    """
+    # iterable is walked only once; tee handles the intermediate storage.
+    t1, t2 = tee(iterable)
+    return filterfalse(pred, t1), filter(pred, t2)
