@@ -1,12 +1,14 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """Miscellaneous lispy constructs."""
 
-__all__ = ["call", "raisef", "pack", "namelambda"]
+__all__ = ["call", "callwith", "raisef", "pack", "namelambda", "Box"]
 
 from types import LambdaType
 import re
 
+from .regutil import register_decorator
+
+@register_decorator(priority=80)
 def call(f, *args, **kwargs):
     """Call the function f.
 
@@ -18,7 +20,8 @@ def call(f, *args, **kwargs):
         Useful for making lispy not-quite-functions where the def just delimits
         a block of code that runs immediately (think call-with-something in Lisps).
 
-        In this case, the function will be called with zero arguments.
+        The function will be called with no arguments. If you need to pass
+        arguments when using ``call`` as a decorator, see ``callwith``.
 
     **When called normally**:
 
@@ -74,6 +77,106 @@ def call(f, *args, **kwargs):
     """
     return f(*args, **kwargs)
 
+@register_decorator(priority=80)
+def callwith(*args, **kwargs):
+    """Freeze arguments, choose function later.
+
+    **Used as decorator**, this is like ``@call``, but with arguments::
+
+        @callwith(3)
+        def result(x):
+            return x**2
+        assert result == 9
+
+    **Called normally**, this creates a function to apply the given arguments
+    to a callable to be specified later::
+
+        def myadd(a, b):
+            return a + b
+        def mymul(a, b):
+            return a * b
+        apply23 = callwith(2, 3)
+        assert apply23(myadd) == 5
+        assert apply23(mymul) == 6
+
+    When called normally, the two-step application is mandatory. The first step
+    stores the given arguments. It returns a function ``f(callable)``. When
+    ``f`` is called, it calls its ``callable`` argument, passing in the arguments
+    stored in the first step.
+
+    In other words, ``callwith`` is similar to ``functools.partial``, but without
+    specializing to any particular function. The function to be called is
+    given later, in the second step.
+
+    Hence, ``callwith(2, 3)(myadd)`` means "make a function that passes in
+    two positional arguments, with values ``2`` and ``3``. Then call this
+    function for the callable ``myadd``".
+
+    But if we instead write``callwith(2, 3, myadd)``, it means "make a function
+    that passes in three positional arguments, with values ``2``, ``3`` and
+    ``myadd`` - not what we want in the above example.
+
+    Curry obviously does not help; it will happily pass in all arguments
+    in one go. If you want to specialize some arguments now and some later,
+    use ``partial``::
+
+        from functools import partial
+
+        p1 = partial(callwith, 2)
+        p2 = partial(p1, 3)
+        p3 = partial(p2, 4)
+        apply234 = p3()  # actually call callwith, get the function
+        def add3(a, b, c):
+            return a + b + c
+        def mul3(a, b, c):
+            return a * b * c
+        assert apply234(add3) == 9
+        assert apply234(mul3) == 24
+
+    If the code above feels weird, it should. Arguments are gathered first,
+    and the function to which they will be passed is chosen in the last step.
+
+    A pythonic alternative to the above examples is::
+
+        a = [2, 3]
+        def myadd(a, b):
+            return a + b
+        def mymul(a, b):
+            return a * b
+        assert myadd(*a) == 5
+        assert mymul(*a) == 6
+
+        a = [2]
+        a += [3]
+        a += [4]
+        def add3(a, b, c):
+            return a + b + c
+        def mul3(a, b, c):
+            return a * b * c
+        assert add3(*a) == 9
+        assert mul3(*a) == 24
+
+    Another use case of ``callwith`` is ``map``, if we want to vary the function
+    instead of the data::
+
+        m = map(callwith(3), [lambda x: 2*x, lambda x: x**2, lambda x: x**(1/2)])
+        assert tuple(m) == (6, 9, 3**(1/2))
+
+    The pythonic alternative here is to use the comprehension notation,
+    which can already do this::
+
+        m = (f(3) for f in [lambda x: 2*x, lambda x: x**2, lambda x: x**(1/2)])
+        assert tuple(m) == (6, 9, 3**(1/2))
+
+    Inspiration:
+
+        *Function application with $* in
+        http://learnyouahaskell.com/higher-order-functions
+    """
+    def applyfrozenargsto(f):
+        return f(*args, **kwargs)
+    return applyfrozenargsto
+
 def raisef(exctype, *args, **kwargs):
     """``raise`` as a function, to make it possible for lambdas to raise exceptions.
 
@@ -142,59 +245,35 @@ def namelambda(function, name):
     This is used by ``env``, and by the ``namedlambda`` macro.
     """
     if isinstance(function, LambdaType) and function.__name__ == "<lambda>":
-        myname = "{} (lambda)".format(name)
+        myname = "{}".format(name)
         function.__name__ = myname
         function.__qualname__ = re.sub("<lambda>$", myname, function.__qualname__)
     return function
 
-def test():
-    # def as a code block (function overwritten by return value)
-    #
-    @call
-    def result():
-        return "hello"
-    assert result == "hello"
+class Box:
+    """Minimalistic, mutable single-item container à la Racket.
 
-    # use case 1: make temporaries fall out of scope
-    @call
-    def x():
-        a = 2  #    many temporaries that help readability...
-        b = 3  # ...of this calculation, but would just pollute locals...
-        c = 5  # ...after the block exits
-        return a * b * c
-    assert x == 30
+    Motivation::
 
-    # use case 2: multi-break out of nested loops
-    @call
-    def result():
-        for x in range(10):
-            for y in range(10):
-                if x * y == 42:
-                    return (x, y)
-                ... # more code here
-    assert result == (6, 7)
+        v = 17
+        def f(v):
+            v = 23  # no!
+        f(v)
+        print(v)  # still 17
 
-    from operator import add
-    assert call(add, 2, 3) == add(2, 3)
+    Solution - box it, to keep the actual data in an attribute::
 
-    l = lambda: raisef(ValueError, "all ok")
-    try:
-        l()
-    except ValueError:
-        pass
-    else:
-        assert False
+        b = Box(17)
+        def f(b):
+            b.value = 23  # yes!
+        f(b)
+        print(b.value)  # 23
 
-    myzip = lambda lol: map(pack, *lol)
-    lol = ((1, 2), (3, 4), (5, 6))
-    assert tuple(myzip(lol)) == ((1, 3, 5), (2, 4, 6))
-
-    square = lambda x: x**2
-    assert square.__name__ == "<lambda>"
-    square = namelambda(square, "square")
-    assert square.__name__ == "square (lambda)"
-
-    print("All tests PASSED")
-
-if __name__ == '__main__':
-    test()
+    **Disclaimer**: maybe silly. The standard pythonic solutions are to use
+    a single-item list, or the ``nonlocal`` or ``global`` statements as
+    appropriate for the particular situation.
+    """
+    def __init__(self, value=None):
+        self.value = value
+    def __repr__(self):
+        return "<Box at 0x{:x}, value={}>".format(id(self), self.value)
